from typing import List

from verity_sdk.protocols.Protocol import Protocol
from verity_sdk.utils import COMMUNITY_MSG_QUALIFIER


class CommittedAnswer(Protocol):
    """
    The CommittedAnswer protocol allows one self-sovereign party ask another self-sovereign a question and get a
    answer. The answer can be signed (not required) and must be one of a specified list of responses
    """

    MSG_FAMILY = 'committedanswer'
    """the family name for the message family"""

    MSG_FAMILY_VERSION = '1.0'
    """the version for the message family"""

    # Messages
    ASK_QUESTION = 'ask-question'
    """Name for 'ask-question' control message"""
    GET_STATUS = 'get-status'
    """Name for 'get-status' control message"""
    ANSWER_QUESTION = 'answer-question'
    """Name for 'answer-question' signal message"""
    ANSWER_GIVEN = 'answer-given'
    """Name for 'answer-given' signal message"""

    # pylint:disable=too-many-arguments
    def __init__(self,
                 for_relationship: str,
                 thread_id: str = None,
                 question: str = None,
                 descr: str = '',
                 valid_responses: List[str] = None,
                 signature_required: bool = True,
                 answer_str: str = None,
                 msg_family: str = MSG_FAMILY,  # Used to override the msg_family for the
                 msg_family_version: str = MSG_FAMILY_VERSION,
                 msg_qualifier: str = COMMUNITY_MSG_QUALIFIER):
        """
        Args:
            for_relationship (str): the relationship identifier (DID) for the pairwise relationship that will be used
            thread_id (str): the thread id of the already started protocol
            question (str): The main text of the question (included in the message the Connect.Me user signs with their private key)
            descr (str): Any additional information about the question
            valid_responses (List[str]): The given possible responses.
            signature_required (bool): if a signature must be included with the answer
            answer_str (str): The given answer from the list of valid responses given in the question
        """
        super().__init__(msg_family, msg_family_version, msg_qualifier, thread_id)

        self.for_relationship: str = for_relationship
        self.question: str = question
        self.descr: str = descr
        self.valid_responses: List[str] = valid_responses
        self.signature_required: bool = signature_required
        self.answer_str: str = answer_str

    async def ask(self, context):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
        """
        await self.send_message(context, await self.ask_msg_packed(context))

    def ask_msg(self, context):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.ASK_QUESTION)
        self._add_thread(msg)
        self._add_relationship(msg, self.for_relationship)
        msg['text'] = self.question
        msg['detail'] = self.descr
        msg['valid_responses'] = self.valid_responses or []
        msg['signature_required'] = self.signature_required
        return msg

    async def ask_msg_packed(self, context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.ask_msg(context))

    async def answer(self, context):
        """
        Directs verity-application to answer the question

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
        """
        await self.send_message(context, await self.answer_msg_packed(context))

    def answer_msg(self, context):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.ANSWER_QUESTION)
        self._add_thread(msg)
        self._add_relationship(msg, self.for_relationship)
        msg.response = self.answer_str
        return msg

    async def answer_msg_packed(self, context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.answer_msg(context))

    async def status(self, context):
        """
        Ask for status from the verity-application agent

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
        """
        await self.send_message(context, await self.status_msg_packed(context))

    def status_msg(self, context):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.GET_STATUS)
        self._add_thread(msg)
        self._add_relationship(msg, self.for_relationship)
        return msg

    async def status_msg_packed(self, context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.status_msg(context))



import pytest

from test.test_utils import get_test_config
from verity_sdk.protocols.v1_0.Relationship import Relationship, GoalsList
from verity_sdk.utils import EVERNYM_MSG_QUALIFIER
from verity_sdk.utils.Context import Context

for_relationship = 'RxRJCMe5XNqc9e9J1YPwhL'
thread_id = '7a80285e-896c-45f6-b386-39ed7c49230c'
label = 'test_label'
logo_url = 'logo_url'
phone_number = '+18011234567'
short_invite = False


def test_init():
    relationship = Relationship(
        for_relationship=for_relationship,
        thread_id=thread_id,
        label=label,
        logo_url=logo_url,
    )

    assert relationship.label == label
    assert relationship.for_relationship == for_relationship
    assert relationship.thread_id == thread_id
    assert relationship.logo_url == logo_url

    relationship = Relationship(
        label=None
    )

    assert relationship.label is None
    assert relationship.logo_url is None


@pytest.mark.asyncio
async def test_create():
    context = await Context.create_with_config(await get_test_config())
    relationship = Relationship(
        for_relationship=for_relationship,
        thread_id=thread_id,
        label=label
    )

    msg = relationship.create_msg(context)

    assert msg['@type'] == '{};spec/{}/{}/{}'.format(
        EVERNYM_MSG_QUALIFIER,
        Relationship.MSG_FAMILY,
        Relationship.MSG_FAMILY_VERSION,
        Relationship.CREATE
    )
    assert msg['@id'] is not None
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] is not None
    assert msg['label'] == label
    assert 'logoUrl' not in msg
    assert 'phoneNumber' not in msg


@pytest.mark.asyncio
async def test_create_with_logo_url():
    context = await Context.create_with_config(await get_test_config())
    relationship = Relationship(
        for_relationship=for_relationship,
        thread_id=thread_id,
        label=label,
        logo_url=logo_url
    )

    msg = relationship.create_msg(context)

    assert msg['@type'] == '{};spec/{}/{}/{}'.format(
        EVERNYM_MSG_QUALIFIER,
        Relationship.MSG_FAMILY,
        Relationship.MSG_FAMILY_VERSION,
        Relationship.CREATE
    )
    assert msg['@id'] is not None
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] is not None
    assert msg['label'] == label
    assert msg['logoUrl'] == logo_url
    assert 'phoneNumber' not in msg


@pytest.mark.asyncio
async def test_create_with_phone_number():
    context = await Context.create_with_config(await get_test_config())
    relationship = Relationship(
        for_relationship=for_relationship,
        thread_id=thread_id,
        label=label,
        phone_number=phone_number
    )

    msg = relationship.create_msg(context)

    assert msg['@type'] == '{};spec/{}/{}/{}'.format(
        EVERNYM_MSG_QUALIFIER,
        Relationship.MSG_FAMILY,
        Relationship.MSG_FAMILY_VERSION,
        Relationship.CREATE
    )
    assert msg['@id'] is not None
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] is not None
    assert msg['label'] == label
    assert 'logoUrl' not in msg
    assert msg['phoneNumber'] == phone_number


@pytest.mark.asyncio
async def test_connection_invitation():
    context = await Context.create_with_config(await get_test_config())
    relationship = Relationship(
        for_relationship=for_relationship,
        thread_id=thread_id,
        label=label
    )

    msg = relationship.connection_invitation_msg(context)
    print(msg)

    assert msg['@type'] == '{};spec/{}/{}/{}'.format(
        EVERNYM_MSG_QUALIFIER,
        Relationship.MSG_FAMILY,
        Relationship.MSG_FAMILY_VERSION,
        Relationship.CONNECTION_INVITATION
    )
    assert msg['@id'] is not None
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] is not None
    assert msg['~for_relationship'] == for_relationship
    assert 'shortInvite' not in msg


@pytest.mark.asyncio
async def test_connection_invitation_with_short_invite():
    context = await Context.create_with_config(await get_test_config())
    relationship = Relationship(
        for_relationship=for_relationship,
        thread_id=thread_id
    )

    msg = relationship.connection_invitation_msg(context, short_invite=short_invite)
    print(msg)

    assert msg['@type'] == '{};spec/{}/{}/{}'.format(
        EVERNYM_MSG_QUALIFIER,
        Relationship.MSG_FAMILY,
        Relationship.MSG_FAMILY_VERSION,
        Relationship.CONNECTION_INVITATION
    )
    assert msg['@id'] is not None
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] is not None
    assert msg['~for_relationship'] == for_relationship
    assert msg['shortInvite'] == short_invite


@pytest.mark.asyncio
async def test_sms_connection_invitation():
    context = await Context.create_with_config(await get_test_config())
    relationship = Relationship(
        for_relationship=for_relationship,
        thread_id=thread_id
    )

    msg = relationship.sms_connection_invitation_msg(context)
    print(msg)

    assert msg['@type'] == '{};spec/{}/{}/{}'.format(
        EVERNYM_MSG_QUALIFIER,
        Relationship.MSG_FAMILY,
        Relationship.MSG_FAMILY_VERSION,
        Relationship.SMS_CONNECTION_INVITATION
    )
    assert msg['@id'] is not None
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] is not None
    assert msg['~for_relationship'] == for_relationship


@pytest.mark.asyncio
async def test_out_of_band_invitation():
    context = await Context.create_with_config(await get_test_config())
    relationship = Relationship(
        for_relationship=for_relationship,
        thread_id=thread_id
    )

    msg = relationship.out_of_band_invitation_msg(context)

    assert msg['@type'] == '{};spec/{}/{}/{}'.format(
        EVERNYM_MSG_QUALIFIER,
        Relationship.MSG_FAMILY,
        Relationship.MSG_FAMILY_VERSION,
        Relationship.OUT_OF_BAND_INVITATION
    )
    assert msg['@id'] is not None
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] is not None
    assert msg['~for_relationship'] == for_relationship
    assert msg['goalCode'] == GoalsList.P2P_MESSAGING.value.code
    assert msg['goal'] == GoalsList.P2P_MESSAGING.value.name
    assert 'shortInvite' not in msg


@pytest.mark.asyncio
async def test_out_of_band_invitation_with_goal():
    context = await Context.create_with_config(await get_test_config())
    relationship = Relationship(
        for_relationship=for_relationship,
        thread_id=thread_id
    )

    msg = relationship.out_of_band_invitation_msg(context, goal=GoalsList.ISSUE_VC)

    assert msg['@type'] == '{};spec/{}/{}/{}'.format(
        EVERNYM_MSG_QUALIFIER,
        Relationship.MSG_FAMILY,
        Relationship.MSG_FAMILY_VERSION,
        Relationship.OUT_OF_BAND_INVITATION
    )
    assert msg['@id'] is not None
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] is not None
    assert msg['~for_relationship'] == for_relationship
    assert msg['goalCode'] == GoalsList.ISSUE_VC.value.code
    assert msg['goal'] == GoalsList.ISSUE_VC.value.name
    assert 'shortInvite' not in msg


@pytest.mark.asyncio
async def test_out_of_band_invitation_with_short_invite():
    context = await Context.create_with_config(await get_test_config())
    relationship = Relationship(
        for_relationship=for_relationship,
        thread_id=thread_id
    )

    msg = relationship.out_of_band_invitation_msg(context, short_invite=short_invite)

    assert msg['@type'] == '{};spec/{}/{}/{}'.format(
        EVERNYM_MSG_QUALIFIER,
        Relationship.MSG_FAMILY,
        Relationship.MSG_FAMILY_VERSION,
        Relationship.OUT_OF_BAND_INVITATION
    )
    assert msg['@id'] is not None
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] is not None
    assert msg['~for_relationship'] == for_relationship
    assert msg['goalCode'] == GoalsList.P2P_MESSAGING.value.code
    assert msg['goal'] == GoalsList.P2P_MESSAGING.value.name
    assert msg['shortInvite'] == short_invite


@pytest.mark.asyncio
async def test_sms_out_of_band_invitation():
    context = await Context.create_with_config(await get_test_config())
    relationship = Relationship(
        for_relationship=for_relationship,
        thread_id=thread_id
    )

    msg = relationship.sms_out_of_band_invitation_msg(context)

    assert msg['@type'] == '{};spec/{}/{}/{}'.format(
        EVERNYM_MSG_QUALIFIER,
        Relationship.MSG_FAMILY,
        Relationship.MSG_FAMILY_VERSION,
        Relationship.SMS_OUT_OF_BAND_INVITATION
    )
    assert msg['@id'] is not None
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] is not None
    assert msg['~for_relationship'] == for_relationship
    assert msg['goalCode'] == GoalsList.P2P_MESSAGING.value.code
    assert msg['goal'] == GoalsList.P2P_MESSAGING.value.name


@pytest.mark.asyncio
async def test_sms_out_of_band_invitation_with_goal():
    context = await Context.create_with_config(await get_test_config())
    relationship = Relationship(
        for_relationship=for_relationship,
        thread_id=thread_id
    )

    msg = relationship.sms_out_of_band_invitation_msg(context, goal=GoalsList.ISSUE_VC)

    assert msg['@type'] == '{};spec/{}/{}/{}'.format(
        EVERNYM_MSG_QUALIFIER,
        Relationship.MSG_FAMILY,
        Relationship.MSG_FAMILY_VERSION,
        Relationship.SMS_OUT_OF_BAND_INVITATION
    )
    assert msg['@id'] is not None
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] is not None
    assert msg['~for_relationship'] == for_relationship
    assert msg['goalCode'] == GoalsList.ISSUE_VC.value.code
    assert msg['goal'] == GoalsList.ISSUE_VC.value.name



import json

from indy.did import create_and_store_my_did
from indy.error import IndyError


async def create_new_did(wallet_handle: int, seed: str = None):
    """
    Creates a new generic DID in the given wallet based on a given seed
    Args:
        wallet_handle (int):  handle to an created and opened indy wallet
        seed (str):  a 32 character seed string used to deterministically create a key for the DID
    Returns:
         a DID object based on the DID created in the wallet
    """
    try:
        params = {}
        if seed:
            params['seed'] = seed

        (did, verkey) = await create_and_store_my_did(wallet_handle, json.dumps(params))
        return Did(did, verkey)
    except IndyError as e:
        raise Exception('Unable to create DID with wallet', e)


class Did:
    """
    Simple holder object for a DID
    """
    did: str
    verkey: str

    def __init__(self, did, verkey):
        self.did = did
        self.verkey = verkey



from decimal import Context

from verity_sdk.protocols.Protocol import Protocol
from verity_sdk.utils import COMMUNITY_MSG_QUALIFIER


class OutOfBand(Protocol):
    """
    The OutOfBand protocol allow an interaction intent, connection information and reuse of a relationship.
    """

    MSG_FAMILY = 'out-of-band'
    """the family name for the message family"""
    MSG_FAMILY_VERSION = '1.0'
    """the version for the message family"""

    REUSE = 'reuse'
    """Name for 'reuse' control message"""

    def __init__(self,
                 for_relationship: str = None,
                 invite_url: str = None):
        """
        Args:
            for_relationship (str): the relationship identifier (DID) for the pairwise relationship that will be reused
            invite_url (str): the Out-of-Band invitation url
        """
        super().__init__(
            self.MSG_FAMILY,
            self.MSG_FAMILY_VERSION,
            msg_qualifier=COMMUNITY_MSG_QUALIFIER
        )
        self.for_relationship = for_relationship
        self.invite_url = invite_url

    async def reuse(self, context: Context):
        """
        Direct the verity-application agent to reuse the relationship given.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
        """
        await self.send_message(context, await self.reuse_msg_packed(context))

    def reuse_msg(self, context: Context):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.REUSE)
        self._add_thread(msg)
        self._add_relationship(msg, self.for_relationship)

        msg['inviteUrl'] = self.invite_url
        return msg

    async def reuse_msg_packed(self, context: Context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.reuse_msg(context))



# import pytest
#
# from test.test_utils import get_test_config, cleanup
# from verity_sdk.protocols.v1_0.Connecting import Connecting
# from verity_sdk.utils import EVERNYM_MSG_QUALIFIER
# from verity_sdk.utils.Context import Context
#
# source_id = '12345'
# phone_number = '1234357890'
# include_public_did = True
#
#
# def test_init():
#     connecting = Connecting(source_id, phone_number, include_public_did)
#
#     assert connecting.source_id == source_id
#     assert connecting.phone_number == phone_number
#     assert connecting.include_public_did == include_public_did
#
#
# @pytest.mark.asyncio
# async def test_connect():
#     context = await Context.create_with_config(await get_test_config())
#     connecting = Connecting(source_id, phone_number, include_public_did)
#     msg = connecting.connect_msg(context)
#
#     assert msg['@type'] == '{};spec/{}/{}/{}'.format(
#         EVERNYM_MSG_QUALIFIER,
#         Connecting.MSG_FAMILY,
#         Connecting.MSG_FAMILY_VERSION,
#         Connecting.CREATE_CONNECTION
#     )
#     assert msg['@id'] is not None
#     assert msg['sourceId'] == source_id
#     assert msg['phoneNo'] == phone_number
#     assert msg['includePublicDID'] == include_public_did
#
#     await cleanup(context)
#
#
# @pytest.mark.asyncio
# async def test_status():
#     context = await Context.create_with_config(await get_test_config())
#     connecting = Connecting(source_id, phone_number, include_public_did)
#     msg = connecting.status_msg(context)
#
#     assert msg['@type'] == '{};spec/{}/{}/{}'.format(
#         EVERNYM_MSG_QUALIFIER,
#         Connecting.MSG_FAMILY,
#         Connecting.MSG_FAMILY_VERSION,
#         Connecting.GET_STATUS
#     )
#     assert msg['@id'] is not None
#     assert msg['sourceId'] == source_id
#
#     await cleanup(context)



import logging
from typing import Callable, NewType, Awaitable, Dict

from verity_sdk.utils import Context, unpack_message, MsgType

HandlerFunction = NewType('HandlerFunction', Callable[[str, dict], Awaitable[None]])
DefaultHandlerFunction = NewType('DefaultHandlerFunction', Callable[[dict], Awaitable[None]])


class Handler:
    """
    Defines how to handle a message of a certain type and optionally with a particular status
    """
    def __init__(
            self,
            msg_family: str,
            msg_family_version: str,
            handler_function: HandlerFunction):
        self.msg_family: str = msg_family
        self.msg_family_version: str = msg_family_version
        self.handler_function: HandlerFunction = handler_function

    def handles(self, message: dict) -> bool:

        if '@type' not in message:
            logging.error('message does not contain an "@type" attribute')
            return False
        msg_type = MsgType(message['@type'])
        return msg_type.msg_family == self.msg_family and msg_type.msg_family_version == self.msg_family_version

    async def handle(self, msg_name: str, message: dict):
        """
        Handles the given JSON object message

        Args:
         message (dict): the message to be handled
        """
        await self.handler_function(msg_name, message)

    async def __call__(self, msg_name: str, message: dict):  # Added for clarity and flexibility. Same as `self.handle`.
        await self.handler_function(msg_name, message)


class Handlers:
    """
    Stores an array of message handlers that are used when receiving an inbound message
    """
    _handlers: Dict[str, Handler]
    _default_handler: DefaultHandlerFunction

    def __init__(self):
        self._handlers = {}

    def add_handler(
            self,
            msg_family: str,
            msg_family_version: str,
            handler_function: HandlerFunction):
        """
        Adds a MessageHandler for a message type to the list if current message handlers

        Args:
            msg_family (str): the family of the message to be handled
        msg_family_version (str): the family version to be handled
        handler_function (HandlerFunction):  the handler function itself
        """
        key = Handlers._build_handlers_key(msg_family, msg_family_version)
        self._handlers[key] = Handler(msg_family, msg_family_version, handler_function)

    def has_handler(self, msg_family: str, msg_family_version: str):
        """
        checks list of handlers to see if a handler is register for family

        msg_family (str): the family of the message to be checked
        msg_family_version (str): the family version to be checked

        Returns:
            bool: if there is a handler for given parameters
        """
        return Handlers._build_handlers_key(msg_family, msg_family_version) in self._handlers

    def set_default_handler(self, handler_function: DefaultHandlerFunction):
        """
        Sets a handler to be called for messages where there is no registered handler
        Args:
            handler_function (DefaultHandlerFunction): the function that will be called
        """
        self._default_handler = handler_function

    async def handle_message(self, context: Context, raw_message: bytes):
        """
        Calls all of the handlers that support handling of this particular message type and message status

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
            raw_message (bytes): the raw bytes received from Verity
        """
        message: dict = await unpack_message(context, raw_message)
        msg_type = MsgType(message['@type'])

        if self.has_handler(msg_type.msg_family, msg_type.msg_family_version):
            key = Handlers._build_handlers_key(msg_type.msg_family, msg_type.msg_family_version)
            await self._handlers[key].handle(msg_type.msg_name, message)
        else:
            if self._default_handler is not None:
                await self._default_handler(message)
            else:
                logging.warning('Unable to handle message, and no default handler was defined')

    @staticmethod
    def _build_handlers_key(msg_family: str, msg_family_version: str):
        return '{}{}'.format(msg_family, msg_family_version)

# # Enables handler registration decorator @AddHandler(handlers, msg_family, msg_family_version)
# class AddHandler:
#
#     def __init__(self, handlers: Handlers, msg_family: str, msg_family_version: str,):
#         self.handlers = handlers
#         self.msg_family = msg_family
#         self.msg_family_version = msg_family_version
#
#     def __call__(self, handler_function: HandlerFunction, *args, **kwargs):
#         self.handlers.add_handler(self.msg_family, self.msg_family_version, handler_function)



import pytest

from test.test_utils import get_test_config
from verity_sdk.protocols.v1_0.OutOfBand import OutOfBand
from verity_sdk.utils import COMMUNITY_MSG_QUALIFIER
from verity_sdk.utils.Context import Context


def test_init():
    relationship = OutOfBand(
        for_relationship='RxRJCMe5XNqc9e9J1YPwhL',
        invite_url='test_invite_url'
    )

    assert relationship.for_relationship == 'RxRJCMe5XNqc9e9J1YPwhL'
    assert relationship.invite_url == 'test_invite_url'


@pytest.mark.asyncio
async def test_reuse():
    context = await Context.create_with_config(await get_test_config())
    relationship = OutOfBand(
        for_relationship='RxRJCMe5XNqc9e9J1YPwhL',
        invite_url='test_invite_url'
    )

    msg = relationship.reuse_msg(context)

    assert msg['@type'] == '{};spec/{}/{}/{}'.format(
        COMMUNITY_MSG_QUALIFIER,
        OutOfBand.MSG_FAMILY,
        OutOfBand.MSG_FAMILY_VERSION,
        OutOfBand.REUSE
    )
    assert msg['@id'] is not None
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] is not None
    assert msg['inviteUrl'] == 'test_invite_url'



from indy.error import IndyError, ErrorCode
from indy.wallet import create_wallet, open_wallet


async def create_and_open_wallet(wallet_config, wallet_credentials):
    """
    Tries to create a wallet with the given parameters. If creation fails because the wallet already exists, that
    error is trapped and this function will complete but other errors will be raised. The created or the already
    existing wallet is then opened and the handle is returned.

    Args:
        wallet_config: indy wallet config
        wallet_credentials: indy wallet credential
    Returns:
        int: indy wallet handle
    """
    try:
        await create_wallet(wallet_config, wallet_credentials)
    except IndyError as e:
        if e.error_code != ErrorCode.WalletAlreadyExistsError:
            raise e

    return await open_wallet(wallet_config, wallet_credentials)



from verity_sdk.protocols.Protocol import Protocol
from verity_sdk.utils import EVERNYM_MSG_QUALIFIER


class WriteCredentialDefinition(Protocol):
    """
    The WriteCredentialDefinition protocol writes credential definitions to an Indy Identity Ledger (often the
    Sovrin Ledger). This protocol expect that the issuer has been setup."""

    MSG_FAMILY = 'write-cred-def'
    """the family name for the message family"""

    MSG_FAMILY_VERSION = '0.6'
    """the version for the message family"""

    # Messages
    WRITE_CRED_DEF = 'write'
    """Name for 'write' control message"""

    STATUS = 'status-report'
    """Name for 'status-report' signal message"""

    def __init__(self, name: str, schema_id: str, tag: str = None, revocation_details: dict = None):
        """
        Args:
            name (str): The name of the new credential definition
            schema_id (str): The id of the schema this credential definition will be based on
            tag (str): An optional tag for the credential definition
            revocation_details (dict): the revocation object defining revocation support
        """
        super().__init__(
            self.MSG_FAMILY,
            self.MSG_FAMILY_VERSION,
            msg_qualifier=EVERNYM_MSG_QUALIFIER,
        )
        self.name = name
        self.schema_id = schema_id
        self.tag = tag
        self.revocation_details = revocation_details

    async def write(self, context):
        """
        Directs verity-application to write the specified Credential Definition to the Ledger

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
        """
        await self.send_message(context, await self.write_msg_packed(context))

    def write_msg(self, context):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.WRITE_CRED_DEF)
        msg['name'] = self.name
        msg['schemaId'] = self.schema_id
        msg['tag'] = self.tag
        msg['revocationDetails'] = self.revocation_details
        self._add_thread(msg)
        return msg

    async def write_msg_packed(self, context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.write_msg(context))
